<?php
// Be sure to place adLDAP.php in the same directory as ad-ldap-sso.php
// adLDAP - http://adldap.sourceforge.net
// Tested with adLDAP 4.0.3
// NOTE: NO modifications need to be made to any of the files included with
// the adLDAP distribution.  All required parameters are passed from this 
// script.
include 'config.php';
require_once(dirname(__FILE__) . "/adLDAP.php");

$adldap = new adLDAP(array('base_dn'=>$AD_SSO_base_dn, 'account_suffix'=>$AD_SSO_account_suffix, 'domain_controllers'=>array(0=>$AD_SSO_domain_controllers), 'admin_username'=>$AD_SSO_admin_username, 'admin_password'=>$AD_SSO_admin_password));

// Error log class
// See: http://www.php.net/manual/en/function.error-log.php#86651

function AD_SSO_Log($msg){
                $date = date('Y.m.d h:i:s');
                global $AD_SSO_Log_Path;
                error_log("Date:  ".$date."   |   ".$msg."\n", 3, $AD_SSO_Log_Path);
 }

// Error handling function
function AD_SSO_Error($errortext){
                $failureMessage =  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
                $failureMessage .= "<loginshare>";
                $failureMessage .= "  <result>0</result>";
                $failureMessage .= "  <message>AD SSO: ".$errortext."</message>";
                $failureMessage .= "</loginshare>"; 
                echo $failureMessage;
                global $AD_SSO_Logging;
                if (!empty($AD_SSO_Logging)) {
                        AD_SSO_Log("Failure.  XML output was:");
                        AD_SSO_Log($failureMessage);
                }
}
// Function to parse the AD lookup results
function AD_SSO_Lookup($user,$usergroup,$useracct){
        // A little error checking
        // First, let's make sure adLDAP.php returned something.  If not, we need to send an error 
        // response to LoginShare.
        if(!$user) AD_SSO_Error("Error - Invalid username, or no response from Active Directory.");

        // Since Kayako associates users by means of email address, we need to make sure we've been
        // able to pull this from AD.  If we were able to get an email address, let's go ahead and 
        // create the XML response for LoginShare.
        // Note - the proxyAddresses we're pulling from adLDAP are *Exchange* email addresses.  
        elseif(is_array($user->proxyAddresses)) {
                $emailArray = $user->proxyAddresses;
                $successMessage = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
                $successMessage .= "<loginshare>";
                $successMessage .= "  <result>1</result>";
                $successMessage .= "  <user>";
                $successMessage .= "          <usergroup>".$usergroup."</usergroup>";
                $successMessage .= "          <fullname>" . $user->displayName . "</fullname>";
                $successMessage .= "          <designation>" . $user->title . "</designation>";
                // A user may have more than one email address in AD.  Let's be sure to tell
                // Kayako about all of them.
                $successMessage .= "          <emails>";
                foreach($emailArray as $email) {
                        if(stristr($email, 'smtp:')) { // Find just SMTP addresses
                                $email_stripped=preg_replace("/\b(smtp:)/i","",$email); // And remove the SMTP: or smtp: prefix
                                $successMessage .= "                  <email>".$email_stripped."</email>";
                        }
                }
                $successMessage .= "          </emails>";
                $successMessage .= "          <phone>".$user->telephonenumber."</phone>";
                $successMessage .= "  </user>";   
                $successMessage .= "</loginshare>";      
                echo $successMessage;          
                // If logging is enabled, send the output to the log as well.
                global $AD_SSO_Logging;
                if (!empty($AD_SSO_Logging)) {
                        AD_SSO_Log("Success.  XML output was:");
                        AD_SSO_Log($successMessage);
                }
            }

        // If adLDAP.php returned user information, but no email address was found, let's throw an error.  This would be the case with a non (Exchange) mail-enabled user.
        else AD_SSO_Error("Error - No email address found for account: ".$useracct);
}

// Make sure we received a value for username & password via POST
if (!isset($_POST["username"]) || !isset($_POST["password"])) AD_SSO_Error("Error - No username or no password was received"); 
               
else {
        $username=$_POST["username"];
        $password=$_POST["password"];
        // If logging is enabled, send the output to the log as well.
                if (!empty($AD_SSO_Logging)) {
                        AD_SSO_Log("_POST username: ".$_POST["username"]);
                        AD_SSO_Log("_POST password: ".$_POST["password"]);
                }
        // Check to make sure the email address received was not tampered with, and the same as what was
        // calculated by login.php.
        // If this checks out OK, then we're going to call adldap, then send the results to our AD_SSO_Lookup
        // function to be parsed.
        if ($password==hash_hmac('sha256', $username, $AD_SSO_shared_secret)){
                $uname = $adldap->user()->infoCollection($username, array("proxyAddresses","displayName","telephonenumber","title"));
                $usergroup=$AD_SSO_Fusion_Group;
                AD_SSO_Lookup($uname,$usergroup,$username);
        }
        // If the email address doesn't match the hashed value computed by login.php, then display an error
        else AD_SSO_Error("Error - Security error for user ".$username);                
}
?>